import _extends from "@babel/runtime/helpers/esm/extends";
import { useContext, useMemo } from 'react';
import PdeContext from '../contexts/PdeContext';
var isNode = typeof window === 'undefined';
var serverStrategy = {
  getVariation: function getVariation(_ref) {
    var pde = _ref.pde,
        experimentName = _ref.experimentName,
        attributes = _ref.attributes;
    return pde.getVariation({
      pde: pde,
      name: experimentName,
      attributes: attributes
    });
  },
  trackExperiment: function trackExperiment() {}
};
var browserStrategy = {
  getVariation: function getVariation(_ref2) {
    var pde = _ref2.pde,
        experimentName = _ref2.experimentName,
        attributes = _ref2.attributes;
    var variationName = pde.activateExperiment({
      name: experimentName,
      attributes: attributes
    });
    return variationName;
  },
  trackExperiment: function trackExperiment(_ref3) {
    var _window$analytics;

    var variationName = _ref3.variationName,
        experimentName = _ref3.experimentName;
    // user is not part of the experiment
    if (!variationName) return;

    if (!((_window$analytics = window.analytics) != null && _window$analytics.track)) {
      // eslint-disable-next-line no-console
      console.error("[sui-pde: useExperiment] window.analytics.track expected to exists but doesn't");
      return;
    }

    window.analytics.track('Experiment Viewed', {
      experimentName: experimentName,
      variationName: variationName
    });
  }
};
/**
 * Hook to use a experiment
 * @param {object} param
 * @param {string} param.experimentName
 * @param {object} param.attributes
 * @param {function} param.trackExperimentViewed
 * @return {{variation: string}}
 */

export default function useExperiment(_temp) {
  var _ref4 = _temp === void 0 ? {} : _temp,
      experimentName = _ref4.experimentName,
      attributes = _ref4.attributes,
      trackExperimentViewed = _ref4.trackExperimentViewed;

  var _useContext = useContext(PdeContext),
      pde = _useContext.pde;

  if (pde === null) throw new Error('[sui-pde: useExperiment] sui-pde provider is required to work');
  var variation = useMemo(function () {
    var variationName;

    var clientStrategy = _extends({}, browserStrategy, trackExperimentViewed && {
      trackExperiment: trackExperimentViewed
    });

    var strategy = isNode ? serverStrategy : clientStrategy;

    try {
      variationName = strategy.getVariation({
        pde: pde,
        experimentName: experimentName,
        attributes: attributes
      });
      strategy.trackExperiment({
        variationName: variationName,
        experimentName: experimentName
      });
    } catch (error) {
      // eslint-disable-next-line no-console
      console.error(error);
      return null;
    }

    return variationName;
  }, [experimentName, pde, attributes, trackExperimentViewed]);
  return {
    variation: variation
  };
}